@@ -1,6 +1,6 @@
-
 #include <qpdf/Pl_StdioFile.hh>
-
+#include <qpdf/QUtil.hh>
+#include <stdexcept>
 #include <errno.h>
 
 Pl_StdioFile::Pl_StdioFile(char const* identifier, FILE* f) :
@@ -22,8 +22,8 @@ Pl_StdioFile::write(unsigned char* buf, int len)
 	so_far = fwrite(buf, 1, len, this->file);
 	if (so_far == 0)
 	{
-	    throw QEXC::System(this->identifier + ": Pl_StdioFile::write",
-			       errno);
+	    QUtil::throw_system_error(
+		this->identifier + ": Pl_StdioFile::write");
 	}
 	else
 	{
@@ -42,7 +42,8 @@ Pl_StdioFile::finish()
     }
     else
     {
-	throw QEXC::Internal(this->identifier +
-			     ": Pl_StdioFile::finish: stream already closed");
+	throw std::logic_error(
+	    this->identifier +
+	    ": Pl_StdioFile::finish: stream already closed");
     }
 }
